
#include <bits/stdc++.h>

#pragma GCC optimize "-O3"

#define endl '\n'
#define trace(x) cerr << #x << " = " << x << endl

using namespace std;

struct Node {
    int minValue;
    int maxValue;
    int lazy;

    Node(int minValue = 0, int maxValue = 0) {
        this->minValue = minValue;
        this->maxValue = maxValue;
        this->lazy = 0;
    }

    Node operator +(Node rhs) {
        return Node(min(this->minValue, rhs.minValue), max(this->maxValue, rhs.maxValue));
    }
};

struct Query {
    int type;
    int l;
    int r;
    int t;

    Query(int type, int l, int r, int t) {
        this->type = type;
        this->l = l;
        this->r = r;
        this->t = t;
    }
};

void setIO() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
}

int n, q;

void read() {
    cin >> n >> q;
}

vector<vector<Node>> tree;

void push(int index, int node, int low, int high) {
    if (tree[index][node].lazy) {
        tree[index][node].minValue += tree[index][node].lazy;
        tree[index][node].maxValue += tree[index][node].lazy;

        if (low != high) {
            tree[index][node << 1].lazy += tree[index][node].lazy;
            tree[index][node << 1 | 1].lazy += tree[index][node].lazy;
        }

        tree[index][node].lazy = 0;
    }
}

void update(int index, int node, int low, int high, int ql, int qr) {
    push(index, node, low, high);

    if (high < ql || low > qr) {
        return;
    }

    if (low >= ql && high <= qr) {
        tree[index][node].lazy++;
        push(index, node, low, high);

        return;
    }

    int mid = low + ((high - low) >> 1);
    update(index, node << 1, low, mid, ql, qr);
    update(index, node << 1 | 1, mid + 1, high, ql, qr);

    tree[index][node] = tree[index][node << 1] + tree[index][node << 1 | 1];
}

Node query(int index, int node, int low, int high, int ql, int qr) {
    push(index, node, low, high);

    if (high < ql || low > qr) {
        return Node(1e9, -1e9);
    }

    if (low >= ql && high <= qr) {
        return tree[index][node];
    }

    int mid = low + ((high - low) >> 1);
    return query(index, node << 1, low, mid, ql, qr) + query(index, node << 1 | 1, mid + 1, high, ql, qr);
}

inline void add(int l, int r, int type) {
    update(type, 1, 0, n - 1, l, r);
}

bool query(int l, int r) {
    for (int i = 0; i < (int)tree.size(); i++) {
        Node answer = query(i, 1, 0, n - 1, l, r);

        if (answer.minValue != answer.maxValue) {
            return false;
        }
    }

    return true;
}

void solve() {
    vector<Query> queries;

    for (int i = 0; i < q; i++) {
        int type, l, r;
        cin >> type >> l >> r;

        l--;
        r--;

        if (type == 1) {
            int t;
            cin >> t;

            queries.emplace_back(type, l, r, t);
        } else {
            queries.emplace_back(type, l, r, -1);
        }
    }

    vector<int> compT;

    for (int i = 0; i < q; i++) {
        if (queries[i].type == 1) {
            compT.emplace_back(queries[i].t);
        }
    }

    sort(compT.begin(), compT.end());
    compT.resize(unique(compT.begin(), compT.end()) - compT.begin());

    for (int i = 0; i < q; i++) {
        if (queries[i].type == 1) {
            queries[i].t = lower_bound(compT.begin(), compT.end(), queries[i].t) - compT.begin();
        }
    }

    tree.resize(compT.size(), vector<Node>(n << 2));

    for (int i = 0; i < q; i++) {
        if (queries[i].type == 1) {
            add(queries[i].l, queries[i].r, queries[i].t);
        } else {
            cout << query(queries[i].l, queries[i].r) << endl;
        }
    }
}

int main() {
    setIO();

    read();

    solve();
}
