
#include <bits/stdc++.h>

#pragma GCC optimize "-O3"

#define endl '\n'
#define trace(x) cerr << #x << " = " << x << endl

using namespace std;

struct Node {
    int minValue;
    int maxValue;
    int lazy;
    Node *left, *right;

    Node(int minValue = 0, int maxValue = 0) {
        this->minValue = minValue;
        this->maxValue = maxValue;
        this->lazy = 0;
        this->left = this->right = nullptr;
    }
};

void setIO() {
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
    cout.tie(nullptr);
}

int n, q;

void read() {
    cin >> n >> q;
}

map<int, Node*> trees;

Node* getNode(Node* &node) {
    if (!node) {
        node = new Node();
    }

    return node;
}

void push(Node *node, int low, int high) {
    if (node->lazy > 0) {
        node->minValue += node->lazy;
        node->maxValue += node->lazy;

        if (low != high) {
            getNode(node->left)->lazy += node->lazy;
            getNode(node->right)->lazy += node->lazy;
        }

        node->lazy = 0;
    }
}

void update(Node *&node, int low, int high, int ql, int qr, int delta) {
    node = getNode(node);

    push(node, low, high);

    if (high < ql || low > qr) {
        return;
    }

    if (low >= ql && high <= qr) {
        node->lazy += delta;
        push(node, low, high);

        return;
    }

    int mid = low + ((high - low) >> 1);
    update(node->left, low, mid, ql, qr, delta);
    update(node->right, mid + 1, high, ql, qr, delta);

    node->minValue = min(node->left->minValue, node->right->minValue);
    node->maxValue = max(node->left->maxValue, node->right->maxValue);
}

Node query(Node *&node, int low, int high, int ql, int qr) {
    node = getNode(node);

    push(node, low, high);

    if (high < ql || low > qr) {
        return Node(1e9, -1e9);
    }

    if (low >= ql && high <= qr) {
        return (*node);
    }

    int mid = low + ((high - low) >> 1);
    Node l = query(node->left, low, mid, ql, qr);
    Node r = query(node->right, mid + 1, high, ql, qr);

    Node result(min(l.minValue, r.minValue), max(l.maxValue, r.maxValue));
    return result;
}

void update(int l, int r, int type) {
    update(trees[type], 0, n - 1, l, r, 1);
}

bool query(int l, int r) {
    for (auto &i : trees) {
        Node answer = query(i.second, 0, n - 1, l, r);

        if (answer.minValue != answer.maxValue) {
            return false;
        }
    }

    return true;
}

void solve() {
    while (q--) {
        int type, l, r;
        cin >> type >> l >> r;

        l--;
        r--;

        if (type == 1) {
            int t;
            cin >> t;

            update(l, r, t);
        } else {
            cout << query(l, r) << endl;
        }
    }
}

int main() {
    setIO();

    read();

    solve();
}
