#include <iostream>
#include <vector>
#include <random>

using namespace std;

mt19937 rnd(22);
const int MAXN = 2e5 + 5;

int randInRange(int l, int r)
{
    return l + rnd()%(r-l+1);
}

struct Hash
{
    long long h1, h2;

    Hash(){}
    Hash(long long h1, long long h2)
    {
        this->h1 = h1;
        this->h2 = h2;
    }
};

Hash operator +(Hash A, Hash B)
{
    return Hash(A.h1+B.h1, A.h2+B.h2);
}

Hash operator *(Hash A, long long x)
{
    return Hash(A.h1*x, A.h2*x);
}

bool operator ==(Hash A, Hash B)
{
    if(A.h1!=B.h1) return false;
    if(A.h2!=B.h2) return false;

    return true;
}

bool operator !=(Hash A, Hash B)
{
    if(A.h1!=B.h1) return true;
    if(A.h2!=B.h2) return true;

    return false;
}

struct SegmentTree
{
    int l, r;
    Hash sum, lazy;

    SegmentTree *L, *R;

    SegmentTree()
    {
        this->L = nullptr;
        this->R = nullptr;
        this->sum = Hash(0, 0);
        this->lazy = Hash(0, 0);
    }
    SegmentTree(int l, int r) : SegmentTree()
    {
        this->l = l;
        this->r = r;
    }

    void updateLazy()
    {
        sum = sum + (lazy*(r-l+1));
        if(l!=r)
        {
            L->lazy = L->lazy + lazy;
            R->lazy = R->lazy + lazy;
        }

        lazy = Hash(0, 0);
    }

    void recalc()
    {
        sum = L->sum + R->sum;
    }

    void build()
    {
        if(l==r) return;

        L = new SegmentTree(l, (l+r)/2);
        R = new SegmentTree((l+r)/2+1, r);

        L->build();
        R->build();
    }

    void update(int ql, int qr, Hash change)
    {
        updateLazy();
        if(l>=ql && r<=qr)
        {
            Hash scaledChange = (change*(r-l+1));
            sum = sum + scaledChange;

            if(l!=r)
            {
                L->lazy = L->lazy + change;
                R->lazy = R->lazy + change;
            }

            return;
        }
        if(r<ql || l>qr) return;

        L->update(ql, qr, change);
        R->update(ql, qr, change);
        recalc();
    }

    Hash query(int ql, int qr)
    {
        updateLazy();
        if(l>=ql && r<=qr) return sum;
        if(r<ql || l>qr) return Hash(0, 0);

        return L->query(ql, qr) + R->query(ql, qr);
    }
};

int n;
long long a[MAXN];

SegmentTree *yoanko;
Hash code[MAXN];

void init()
{
    for(int i = 1;i<=1e5;i++) code[i] = Hash(rnd()%(int)1e7, rnd()%(int)1e7);

    yoanko = new SegmentTree(0, n-1);
    yoanko->build();
}

int main()
{
    ios::sync_with_stdio(false);
    cin.tie(NULL);

    int Q;
    cin >> n >> Q;

    init();

    while(Q--)
    {
        int type, l, r, x;

        cin >> type >> l >> r;
        if(type==1) cin >> x;

        l--;r--;
        if(type==1)
        {
            yoanko->update(l, r, code[x]);
        }
        else
        {
            Hash polinoma = yoanko->query(l, r);

            bool fail = false;
            for(int iter = 0;iter<11;iter++)
            {
                int ind = randInRange(l, r);
                Hash welko = yoanko->query(ind, ind);

                if(polinoma!=(welko*(r-l+1)))
                {
                    fail = true;
                    break;
                }
            }

            cout << (fail^1) << '\n';
        }
    }
}
