#include<iostream>
#include<vector>
using namespace std;

const int MAX_PRIME=1300000;

bool sieve[MAX_PRIME+1];
vector<long long> primes;

void GetPrimes()
{
    for(int i=3; i<=MAX_PRIME; i+=2)
    {
        sieve[i]=true;
    }

    primes.push_back(0);
    sieve[2]=true;
    primes.push_back(2);
    for(int i=3; i<=MAX_PRIME; i+=2)
    {
        if(!sieve[i])
        {
            continue;
        }

        primes.push_back(i);
        for(int j=i*i; j>0 && j<=MAX_PRIME; j+=i)
        {
            sieve[j]=false;
        }
    }

    return;
}

struct node
{
    int from, to;
    bool same;
    long long hash1, hash2;
};

struct result
{
    bool same;
    long long hash1, hash2;
};

const int MAXN=2*1e5;
const long long MOD=1e9+7;
node tree[2*MAXN];

void BuildTree(int n, int index, int from, int to)
{
    tree[index].from=from, tree[index].to=to, tree[index].same=true, tree[index].hash1=1, tree[index].hash2=0;

    if(from==to)
    {
        return;
    }

    BuildTree(n, index*2, from, (from+to)/2);
    BuildTree(n, index*2+1, (from+to)/2+1, to);

    return;
}

void AddTree(int n, int index, int from, int to, int x)
{
    if(tree[index].same==true && tree[index].from==from && tree[index].to==to)
    {
        tree[index].hash1=(tree[index].hash1*primes[x])%MOD;
        tree[index].hash2=(tree[index].hash2+primes[x])%MOD;

        return;
    }

    if(from>tree[index*2].to)
    {
        AddTree(n, index*2+1, from, to, x);
    }
    else if(to<=tree[index*2].to)
    {
        AddTree(n, index*2, from, to, x);
    }
    else
    {
        AddTree(n, index*2, from, tree[index*2].to, x);
        AddTree(n, index*2+1, tree[index*2+1].from, to, x);
    }

    if(tree[index*2].hash1==tree[index*2+1].hash1 && tree[index*2].hash2==tree[index*2+1].hash2)
    {
        tree[index].same=true;
        tree[index].hash1=tree[index*2].hash1;
        tree[index].hash2=tree[index*2].hash2;
    }
    else
    {
        //cout << tree[index].from << " - " << tree[index].to << endl;
        tree[index].same=false;
    }

    return;
}

result GetTree(int n, int index, int from, int to)
{
    //cout << index << " " << from << " " <<  to << " " << tree[index].same << "\n";


    if(tree[index].from==from && tree[index].to==to)
    {
        result r;
        if(tree[index].same)
        {
            r.same=true, r.hash1=tree[index].hash1, r.hash2=tree[index].hash2;
            return r;
        }
        else
        {
            r.same=false;
            return r;
        }
    }

    if(from>tree[index*2].to)
    {
        if(tree[index].same)
        {
            tree[index*2+1].hash1=tree[index].hash1;
            tree[index*2+1].hash2=tree[index].hash2;
        }

        return GetTree(n, index*2+1, from, to);
    }
    else if(to<=tree[index*2].to)
    {
        if(tree[index].same)
        {
            tree[index*2].hash1=tree[index].hash1;
            tree[index*2].hash2=tree[index].hash2;
        }

        return GetTree(n, index*2, from, to);
    }
    else
    {
        if(tree[index].same)
        {
            tree[index*2].hash1=tree[index].hash1;
            tree[index*2].hash2=tree[index].hash2;
            tree[index*2+1].hash1=tree[index].hash1;
            tree[index*2+1].hash2=tree[index].hash2;
        }

        result r1=GetTree(n, index*2, from, tree[index*2].to), r2=GetTree(n, index*2+1, tree[index*2+1].from, to);

        if(r1.same && r2.same && r1.hash1==r2.hash1 && r1.hash2==r2.hash2)
        {
            return r1;
        }
        else
        {
            r1.same=false;
            return r1;
        }
    }
}

int main()
{
    ios_base::sync_with_stdio(false);
    cin.tie(NULL);

    GetPrimes();

    int n, q;
    cin >> n >> q;

    BuildTree(n, 1, 1, n);

    while(q>0)
    {
        int type;
        cin >> type;

        if(type==1)
        {
            int l, r, x;

            cin >> l >> r >> x;

            AddTree(n, 1, l, r, x);
        }
        else
        {
            int l, r;

            cin >> l >> r;

            if(GetTree(n, 1, l, r).same)
            {
                cout << "1\n";
            }
            else
            {
                cout << "0\n";
            }
        }

        q--;
    }

    return 0;
}
